/***************************************************************************
 *   $Id: hash.c,v 1.1.1.1 2011-11-17 16:42:29 scg Exp $
 *                                                                         *
 *   (C)2011 St(u)dio of Computer Games                                    *
 *   Alexander Ozumenko <a.ozumenko@gmail.com>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <string.h>

#include "hash.h"



/* string HASH-function */
static inline unsigned int hash_function(const char *str) {
    unsigned int h = 0;
    for(; *str != '\0'; str++) {
        h = h * 31 + *str;
    }
    return h;
}



/* create HASH table */
inline int hash_init(struct hash *h, int m) {
    int i;

    h->table = malloc(m * sizeof(struct hash_entry));
    if(h->table == NULL)
        return -1;
    h->m = m;
    for(i = 0; i < h->m; i++) {
        INIT_LIST_HEAD(&h->table[i]);
    }

    return 0;
}


/* clean and destroy HASH-table */
inline void hash_release(struct hash *h, struct list_head *head) {
    hash_clean(h, head);
    free(h->table);
}


/* clean HASH-table */
inline void hash_clean(struct hash *h, struct list_head *head) {
    int i;

    for(i = 0; i < h->m; i++) {
        while(!list_empty(&h->table[i])) {
#ifndef HASH_STATIC_KEY
            struct hash_entry *entry;
            entry = list_entry(h->table[i].next, struct hash_entry, list);
            free(entry->key);
#endif /* HASH_STATIC_KEY */
            list_move(h->table[i].next, head);
        }
    }
}


/* add entry to hash */
inline int hash_add(struct hash *h, const char *key, struct hash_entry *entry) {
    list_add(&entry->list, &h->table[hash_function(key) % h->m]);
#ifndef HASH_STATIC_KEY
    entry->key = strdup(key);
    if(entry->key == NULL) {
        return -1;
    }
#else
    entry->key = key;
#endif /* HASH_STATIC_KEY */

    return 0;
}


/* get entry from hash */
inline struct hash_entry *hash_get(struct hash *h, const char *key) {
    struct list_head *head, *pos;
    struct hash_entry *entry;

    head = &h->table[hash_function(key) % h->m];
    list_for_each(pos, head) {
        entry = list_entry(pos, struct hash_entry, list);
        if(strcmp(key, entry->key) == 0) return entry;
    }
    return NULL;
}


/* remove entry from hash */
inline struct hash_entry *hash_del(struct hash *h, const char *key) {
    struct list_head *pos;
    struct hash_entry *entry;

    list_for_each(pos, &h->table[hash_function(key) % h->m]) {
        entry = list_entry(pos, struct hash_entry, list);
        if(strcmp(key, entry->key) == 0) {
            list_del(pos);
            return entry;
        }
    }
    return NULL;
}
