/***************************************************************************
 *   $Id: variable.h,v 1.1.1.1 2011-11-17 16:42:29 scg Exp $
 *                                                                         *
 *   (C)2011 St(u)dio of Computer Games                                    *
 *   Alexander Ozumenko <a.ozumenko@gmail.com>                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __VARIABLE_H_
#define __VARIABLE_H_

#include "list.h"
#include "hash.h"



#define VARIABLE_PAGE               64
#define VARIABLE_HASH_SIZE          256



enum VARIABLE_TYPE {
    VARIABLE_TYPE_SCALAR = 0,
    VARIABLE_TYPE_ARRAY,
    VARIABLE_TYPE_HASH,
    VARIABLE_TYPE_MAX
};


struct enviroment {
    /* ToDo: make a list of allocated variables */

    /* unused variables */
    struct list_head var_free[VARIABLE_TYPE_MAX];

    /* root hash */
    struct variable *root;

    int error;
} enviroment;


struct variable {
    /* placement */
    struct enviroment *env;
    union {
        struct list_head list;
        struct hash_entry hash;
    } placement;

    /* value */
    enum VARIABLE_TYPE type;
    union {
        struct {
            char *value;
        } scalar;
        struct array {
            struct list_head list;
        } array;
        struct {
            struct hash hash;
        } hash;
    };
};



int enviroment_init(struct enviroment *env);
void enviroment_release(struct enviroment *env);
struct variable *enviroment_get_root(struct enviroment *env);
int enviroment_errno(struct enviroment *env);

void variable_release(struct variable *var);

enum VARIABLE_TYPE variable_type(struct variable *var);

struct variable *variable_scalar(struct enviroment *env, const char *value);
const char *variable_scalar_get(struct variable *var);

struct variable *variable_array(struct enviroment *env, ...);
struct variable *variable_array_push(struct variable *var, ...);

struct variable *variable_hash(struct enviroment *env, ...);
struct variable *variable_hash_set(struct variable *var, ...);
struct variable *variable_hash_get(struct variable *var, const char *key);



#endif /* __VARIABLE_H_ */
